
//Weather_light_control.ino
// Project: Weather station using DHT22 and device control with touch screen
//--------------------------------------------------------------------------
// Requires the following Arduino libraries:
// - DHT Sensor Library
// - Adafruit Unified Sensor
// - SPFD5408 by JoaoLopesF 

//----------------------------------------Include Library
#include "DHT.h"                      // DHT22 header
#include <SPFD5408_Adafruit_GFX.h>    // Adafruit graphics library
#include <SPFD5408_Adafruit_TFTLCD.h> // Hardware-specific library
#include <SPFD5408_TouchScreen.h>     // Touchscreen library

//----------------------------------------
#if defined(__SAM3X8E__)
    #undef __FlashStringHelper::F(string_literal)
    #define F(string_literal) string_literal
#endif  

//------------------Pin configuration and initialization for Touchscreen
#define YP A2 //--> must be an analog pin, use "An" notation!
#define XM A1 //--> must be an analog pin, use "An" notation!
#define YM 6  //--> can be a digital pin
#define XP 7  //--> can be a digital pin

TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300); 

//--------------------Pin configuration and initialization for LCD TFT
#define LCD_CS A3 
#define LCD_CD A2 
#define LCD_WR A1 
#define LCD_RD A0
// optional
#define LCD_RESET A4

Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);

//----------------------------------------Defines colors
// Assign human-readable names to some common 16-bit color values:
#define BLACK       0x0000
#define BLUE        0x001F
#define RED         0xF800
#define GREEN       0x07E0
#define CYAN        0x07FF
#define MAGENTA     0xF81F
#define YELLOW      0xFFE0
#define WHITE       0xFFFF
#define ORANGE      0xFD20
#define DARKORANGE  0xFB60
#define MAROON      0x7800
#define BLACKM      0x18E3

//----------------- Calibrated values for touch points for LIGHT on/off

#define TS_MINX 500
#define TS_MINY 125
#define TS_MAXX 887
#define TS_MAXY 855

//------------------- The results of the x and y point calibration on the touch screen
//#define TS_MINX 158
//#define TS_MINY 127
//#define TS_MAXX 895
//#define TS_MAXY 872

//------------------Variable for detecting touch screen when touched
#define MINPRESSURE 10
#define MAXPRESSURE 1000

//---------------Defines an LIGHT pin
#define DEVICE 11

//-------------------Button location at point x and y
int BtnGreenX = 30;
int BtnGreenY = 30;
int BtnRedX = 200;
int BtnRedY = 30;

//---------------DHT2 configuration and data variables
#define DHTPIN A5     
#define DHTTYPE DHT22   
DHT dht22(DHTPIN, DHTTYPE);

int h;
float t;
float f;
float hif;
float hic;

//-----------------The x and y points for the Temperature bar/bottle
int x_bar_t = 20;
int y_bar_t = 60;

//-----------------The variable to hold the conversion value from the temperature value to the value for the temperature bar
int T_to_Bar;


//----------------Menu = 0 to display the Main Menu Display, Menu = 1 to control the device and Menu = 2 to display DHT22 sensor data
int Menu = 0;

//--------------------Variable for the x, y and z points on the touch screen
int TSPointZ;
int x_set_rotatoon_135;
int y_set_rotatoon_135;

//-----------------------Millis variable to update the temperature and humidity values
unsigned long previousMillis = 0; //--> will store last time updated
// constants won't change:
const long interval = 2000; //--> interval 

//=======================VOID SETUP()
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  tft.reset();

  // *** SPFD5408 change -- Begin
//  uint16_t identifier = tft.readID();

// -------------------------
//  tft.begin(identifier);

  tft.begin(0x9341); //--> SPFD5408/ILI9341

  tft.setRotation(135); //--> Need for the Mega.  This code rotates the screen. 0=0 , 1=90, 2=180, 3=270

  tft.fillScreen(WHITE); //--> TFT LCD background color

  pinMode(13, OUTPUT);

  pinMode(DEVICE, OUTPUT);

  dht22.begin();

  Menu_display();
}

//===============VOID LOOP()
void loop() {
  // put your main code here, to run repeatedly:

  //----------------------------------------Main Menu Display
  if (Menu == 0) {
    GetTSPoint();

 //----------------------------------------Conditions for detecting touch screen when touched
    if (TSPointZ > MINPRESSURE && TSPointZ < MAXPRESSURE) {

  //---------------Conditions for detecting when the Button for controlling the device is touched and its command (Enter the device controlling menu)
 
// Touch points for Control the light
if (x_set_rotatoon_135 > 10 && x_set_rotatoon_135 < (270) && y_set_rotatoon_135 > 50 && y_set_rotatoon_135 < (90)) {
        Menu = 1;
        DrawButtonControlLEDPress();
        delay(150);
        DrawButtonControlLED();
        delay(500);
        tft.fillScreen(WHITE);
        delay(500);
        DrawButtonGreen(BtnGreenX,BtnGreenY);
        DrawButtonRed(BtnRedX, BtnRedY);
        if (digitalRead(DEVICE) == 0) {
          tft.setTextSize(3);
          tft.setTextColor(BLUE, WHITE);
          tft.setCursor(100, 160);
          tft.print("DEVICE OFF");
        } else if (digitalRead(DEVICE) == 1) {
          tft.setTextSize(3);
          tft.setTextColor(BLUE, WHITE);
          tft.setCursor(100, 160);
          tft.print("DEVICE ON");
        }
        DrawButtonBack(10, 200);
      }
 
     //--------------Condition to detect when the button to display DHT22 sensor data is touched and the command (Enter the menu displays dht22 sensor data)
     // Touch points for weather station
      if (x_set_rotatoon_135 > 70 && x_set_rotatoon_135 < (270) && y_set_rotatoon_135 > -90 && y_set_rotatoon_135 < (-30)) {
        Menu = 2;
        DrawButtonTempHumPress();
        delay(150);
        DrawButtonTempHum();
        delay(500);
        tft.fillScreen(WHITE);
        delay(500);
        tft.drawLine(15, 40, 300, 40, MAROON);
        tft.drawLine(15, 39, 300, 39, MAROON);
      
        tft.setTextSize(2);
        tft.setTextColor(BLUE);
        tft.setCursor(40, 13);
        tft.print("  --Weather Station--");
        
        draw_bar(x_bar_t, y_bar_t);
      
        tft.drawLine(190, 60, 190, 227, MAROON);
        tft.drawLine(190, 127, 300, 127, MAROON);
      
        tft.fillRect(202, 60, 100, 27, CYAN);
        tft.setTextSize(2);
        tft.setTextColor(BLACK);
        tft.setCursor(205, 65);
        tft.print("HUMIDITY");
      
        tft.fillRect(202, 140, 100, 43, GREEN);
        tft.setTextSize(2);
        tft.setTextColor(BLACK);
        tft.setCursor(225, 145); 
        tft.print("TEMP");
        tft.setCursor(210, 165);
        tft.print("C and F");

        DrawButtonBack(8, 6);
      
        GetDHT22Data();
        delay(1000);
      }
      //----------------------------------------
    }  
    //----------------------------------------
  }
  //----------------------------------------

  //----------------------------------------Menu or Mode to control the DEVICE
  if (Menu == 1) {
    ControlTheLED();
  }
  //----------------------------------------

  //----------------------------------------Menu or Mode to display dht22 sensor data
  if (Menu == 2) {
    ShowDHT22Data();
  }
  //----------------------------------------
}

//===========================GetTSPoint()
void GetTSPoint() {
  digitalWrite(13, HIGH);
  TSPoint p = ts.getPoint();
  digitalWrite(13, LOW);

  // if sharing pins, you'll need to fix the directions of the touchscreen pins
  //pinMode(XP, OUTPUT);
  pinMode(XM, OUTPUT);
  pinMode(YP, OUTPUT);
  //pinMode(YM, OUTPUT);

  p.x = map(p.x, TS_MINX, TS_MAXX, 0, tft.height());
  p.y = map(p.y, TS_MINY, TS_MAXY, 0, tft.width());

  y_set_rotatoon_135 = map(p.x, 0, 240, 0, tft.height());
  x_set_rotatoon_135 = map(tft.width() - p.y, 0, 320, 0, tft.width());

  TSPointZ = p.z;
}

//=========================DrawButtonControlLED()
void DrawButtonControlLED() {
  tft.fillRoundRect(17, 90, 280, 40, 10, BLACKM);
  tft.fillRoundRect(19, 92, 276, 36, 10, CYAN);
  tft.setTextSize(2);
  tft.setTextColor(BLACK);
  tft.setCursor(65, 103);  
  tft.print("DEVICE CONTROL");
}

//===========================DrawButtonControlLEDPress()
void DrawButtonControlLEDPress() {
  tft.fillRoundRect(17, 90, 280, 40, 10, BLACKM);
}


//=============================DrawButtonTempHum()
void DrawButtonTempHum() {
  tft.fillRoundRect(17, 160, 280, 40, 10, BLACKM);
  tft.fillRoundRect(19, 162, 276, 36, 10, GREEN);
  tft.setTextSize(2);
  tft.setTextColor(WHITE);
  tft.setCursor(25, 173);
  tft.print("   WEATHER STATION");
}

//===========================DrawButtonTempHumPress()
void DrawButtonTempHumPress() {
  tft.fillRoundRect(17, 160, 280, 40, 10, BLACKM);
}

//=====================DrawButtonGreen(x, y)
void DrawButtonGreen(int xp, int yp) {
  tft.fillRoundRect(xp, yp, 84, 84, 10, BLACKM);
  tft.fillRoundRect(xp+2, yp+2, 80, 80, 10, GREEN);
  tft.setCursor(xp+22, yp+32);
  tft.setTextSize(3);
  tft.setTextColor(WHITE);
  tft.println("ON");
}

//=========================DrawButtonGreenPress(x, y)
void DrawButtonGreenPress(int xp, int yp) {
  tft.fillRoundRect(xp, yp, 84, 84, 10, BLACKM);
}

//=======================DrawButtonRed(x, y)
void DrawButtonRed(int xp, int yp) {
  tft.fillRoundRect(xp, yp, 84, 84, 10, BLACKM);
  tft.fillRoundRect(xp+2, yp+2, 80, 80, 10, RED);
  tft.setCursor(xp+18, yp+32);
  tft.setTextSize(3);
  tft.setTextColor(WHITE);
  tft.println("OFF");
}

//=========================DrawButtonRedPress(x, y)
void DrawButtonRedPress(int xp, int yp) {
  tft.fillRoundRect(xp, yp, 84, 84, 10, BLACKM);
}

//======================DrawButtonBack(x, y)
void DrawButtonBack(int x_btn_back, int y_btn_back) {
  tft.fillRoundRect(x_btn_back, y_btn_back, 60, 30, 10, BLACKM); 
  tft.fillRoundRect(x_btn_back+2, y_btn_back+2, 56, 26, 10, YELLOW);
  tft.setTextSize(2);
  tft.setTextColor(BLACKM);
  tft.setCursor(x_btn_back+7, y_btn_back+7);
  tft.print("Back");  
}

//=====================DrawButtonBackPress(x, y)
void DrawButtonBackPress(int x_btn_back, int y_btn_back) {
  tft.fillRoundRect(x_btn_back, y_btn_back, 30, 30, 5, BLACKM);
}

//=====================Getdht22Data()
void GetDHT22Data() {
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  h = dht22.readHumidity();
  // Read temperature as Celsius (the default)
  t = dht22.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  f = dht22.readTemperature(true);

  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) || isnan(f)) {
    
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  hif = dht22.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  hic = dht22.computeHeatIndex(t, h, false);
}

//======================draw_bar (Temperature Bar)
void draw_bar(int x_bar, int y_bar) {
  tft.fillRoundRect(x_bar, y_bar, 35, 120, 5, BLACK);
  tft.fillCircle(x_bar+17, y_bar+140, 30, BLACK);
  tft.fillRoundRect(x_bar+4, y_bar+4, 27, 120, 2, WHITE);
  tft.fillCircle(x_bar+17, y_bar+140, 25, WHITE);
  tft.fillRect(x_bar+8, y_bar+8, 19, 120, DARKORANGE);
  tft.fillCircle(x_bar+17, y_bar+140, 21, DARKORANGE);

  //tft.fillRect(41, 58, 19, 108, RED);

  tft.drawLine(x_bar+37, y_bar+8, x_bar+42, y_bar+8, RED);
  tft.setTextSize(1);
  tft.setTextColor(RED);
  tft.setCursor(x_bar+47, y_bar+4);
  tft.println("60");
  
  tft.drawLine(x_bar+37, y_bar+115, x_bar+42, y_bar+115, RED);
  tft.setCursor(x_bar+47, y_bar+111);
  tft.println("0");
}

//====================Menu_display()
void Menu_display() {
  tft.setTextSize(3);
  tft.setTextColor(RED);
  tft.setCursor(50, 43);  
  tft.print("MAIN CONSOLE");  

  DrawButtonControlLED();
  DrawButtonTempHum();
}

//====================================ControlTheLED()
void ControlTheLED() {
  GetTSPoint();
  
  if (TSPointZ > MINPRESSURE && TSPointZ < MAXPRESSURE) {
    if (x_set_rotatoon_135 > BtnGreenX && x_set_rotatoon_135 < (BtnGreenX+84) && y_set_rotatoon_135 > BtnGreenY && y_set_rotatoon_135 < (BtnGreenY+84)) {
      tft.setTextSize(3);
      tft.setTextColor(BLUE, WHITE);
      tft.setCursor(100, 160);
      tft.print("DEVICE ON ");

      digitalWrite(DEVICE, HIGH);
      
      DrawButtonGreenPress(BtnGreenX, BtnGreenY);
      delay(100);
      DrawButtonGreen(BtnGreenX,BtnGreenY);
    }

    if (x_set_rotatoon_135 > BtnRedX && x_set_rotatoon_135 < (BtnRedX+84) && y_set_rotatoon_135 > BtnRedY && y_set_rotatoon_135 < (BtnRedY+84)) {
      tft.setTextSize(3);
      tft.setTextColor(BLUE, WHITE);
      tft.setCursor(100, 160);
      tft.print("DEVICE OFF");

      digitalWrite(DEVICE, LOW);
  
      DrawButtonRedPress(BtnRedX, BtnRedY);
      delay(100);
      DrawButtonRed(BtnRedX, BtnRedY);
    }

//    Touch points for back arrow inlight control
      if (x_set_rotatoon_135 > 10 && x_set_rotatoon_135 < (10+30) && y_set_rotatoon_135 > -170 && y_set_rotatoon_135 < (-40)) {
      Menu = 0;
      DrawButtonBackPress(10, 200);
      delay(100);
      DrawButtonBack(10, 200);
      delay(500);
      tft.fillScreen(WHITE);
      delay(500);
      Menu_display();
    }
  }
}

//==============================ShowDHT22Data()
void ShowDHT22Data() {
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    // save the last time you blinked the DEVICE
    previousMillis = currentMillis;
    
    GetDHT22Data();
  }

  T_to_Bar = map(t, 0.0, 50.0, 108, 0);

  tft.fillRect(x_bar_t+8, (y_bar_t+8)+T_to_Bar, 19, 108-T_to_Bar, ORANGE);
  tft.fillRect(x_bar_t+8, y_bar_t+8, 19, T_to_Bar, WHITE);

  tft.setTextSize(2);
  tft.setTextColor(BLACK, GREEN);
  tft.setCursor(75, 100);
  tft.print(t);
  if (t < 10) tft.print(" ");
  tft.setCursor(160, 100);
  tft.print((char)247);
  tft.println("C");

  
  tft.setCursor(75, 135);
  tft.print(f);
  if (f < 100) tft.print(" ");
  tft.setCursor(160, 135);
  tft.print((char)247);
  tft.println("F");

  tft.setTextSize(2);
  tft.setTextColor(BLUE, WHITE);
  tft.setCursor(205, 95);
  tft.print(h);
  tft.print(" RH%");

  tft.setTextSize(2);
  tft.setTextColor(BLACK, WHITE);
  tft.setCursor(205, 200);
  tft.print(hic);
  tft.print(" ");
  tft.print((char)247);
  tft.print("C");
  if (hic < 10) tft.print(" ");

  tft.setTextSize(2);
  tft.setTextColor(BLACK, WHITE);
  tft.setCursor(205, 220);
  tft.print(hif);
  tft.print(" ");
  tft.print((char)247);
  tft.print("F");
  if (hif < 100) tft.print(" "); 

  GetTSPoint();   //Set touch points

  if (TSPointZ > MINPRESSURE && TSPointZ < MAXPRESSURE) {
  //  Touch point settings for back arrow in weather menu
    if (x_set_rotatoon_135 > 1 && x_set_rotatoon_135 < (20) && y_set_rotatoon_135 > 60 && y_set_rotatoon_135 < (290)) {
      Menu = 0;
      DrawButtonBackPress(14, 10);  
      delay(100);
      DrawButtonBack(14, 10); 
      delay(500);
      tft.fillScreen(WHITE);
      delay(500);
      Menu_display();
      return(0);
    }
   }
}
//========================================================================
